@@ -43,7 +43,9 @@ DEBUG_TOOLBAR_CONFIG = {
 
 INTERNAL_IPS = ('127.0.0.12',)
 
-LANGUAGE_CODE = 'en'
+TIME_ZONE = 'Europe/Moscow'
+LANGUAGE_CODE = 'ru'
+
 
 _ = lambda s: s
 
@@ -55,3 +57,5 @@ LANGUAGES = (
 LOCALE_PATHS = (
     os.path.join(PROJECT_ROOT, '../locale'),
 )
+
+#AUTH_CHECK_PASSWORD = False
