@@ -7,8 +7,9 @@ import json
 
 from django.utils.translation import ugettext as _
 from django.core.exceptions import ValidationError
+from django.forms.models import model_to_dict
 from django.contrib.sites.models import Site
-from django.utils.timezone import utc
+from django.utils.timezone import now
 from django.contrib import messages
 from django.db import models
 from django.conf import settings
@@ -16,13 +17,24 @@ from django.conf import settings
 from httpagentparser import detect
 
 from secureauth.utils.token import random_seed, check_seed, get_google_url
+from secureauth.utils import get_ip, get_geo, inet_aton
 from secureauth.utils.codes import RandomPassword
 from secureauth.utils import send_sms, send_mail
-from secureauth.utils import get_ip, get_geo
 from secureauth.utils.sign import Sign
+from secureauth.utils import is_phone
 from secureauth.defaults import (
     SMS_MESSAGE, SMS_CODE_LEN, SMS_AGE, SMS_FROM,
-    CODE_RANGES, SMS_NOTIFICATION_MESSAGE, SMS_NOTIFICATION_SUBJECT)
+    CODE_RANGES, CODE_LEN, SMS_NOTIFICATION_MESSAGE, SMS_NOTIFICATION_SUBJECT,
+    LOGIN_ATTEMPT, BAN_TIME, CHECK_ATTEMPT, CODES_SUBJECT, TOTP_NAME)
+
+
+AUTH_TYPES = (
+    ('', '---'),
+    ('code', _('by code')),
+    ('token', _('by token')),
+    ('phone', _('by sms')),
+    ('question', _('by question')),
+)
 
 
 class UserAuthAbstract(models.Model):
@@ -38,7 +50,7 @@ class UserAuthAbstract(models.Model):
         _('Last verified'), blank=True, null=True, editable=False)
 
     def update_last_verified(self):
-        self.last_verified = datetime.datetime.utcnow().replace(tzinfo=utc)
+        self.last_verified = now()
         self.save(update_fields=['last_verified'])
 
     def check_auth_code(self, auth_code, force_check=False):
@@ -63,7 +75,7 @@ class UserAuthCode(UserAuthAbstract):
         data = {}
         rand = RandomPassword()
         for i in range(1, CODE_RANGES + 1):
-            data[i] = rand.get(max_value=8)
+            data[i] = rand.get(max_value=CODE_LEN)
         self.code = Sign().sign(json.dumps(data))
         return self.save()
 
@@ -90,14 +102,32 @@ class UserAuthCode(UserAuthAbstract):
             number=self.get_code_number()
         )
 
+    @classmethod
+    def send_codes(cls, request):
+        settings_list = cls.objects.filter(user=request.user)
+
+        if settings_list.exists():
+            created_seconds = (now() - settings_list[0].created).seconds
+            if created_seconds > 300:
+                return
+
+            codes = json.loads(Sign().unsign(settings_list[0].code))
+            codes_list = collections.OrderedDict(
+                sorted(codes.items(), key=lambda t: int(t[0])))
+            message = ''
+            for (k, v) in codes_list.items():
+                message += '%s. %s\n' % (k, v)
+            send_mail(
+                [request.user.email], CODES_SUBJECT, message
+            )
+            return True
+
 
 class UserAuthToken(UserAuthAbstract):
     def get_google_url(self):
-        return get_google_url(
-            Sign().unsign(self.code),
-            "%s@%s" % (
-                self.user.get_full_name(), Site.objects.get_current().domain)
-        )
+        data = model_to_dict(Site.objects.get_current())
+        data.update(model_to_dict(self.user))
+        return get_google_url(Sign().unsign(self.code), TOTP_NAME % data)
 
     def _code_is_valid(self, code):
         return check_seed(Sign().unsign(self.code), int(code))
@@ -115,7 +145,7 @@ class UserAuthPhone(UserAuthAbstract):
     phone = models.CharField(_('Phone'), max_length=255, unique=True)
 
     def clean(self):
-        if not self.phone or not self.phone.startswith('+'):
+        if not is_phone(self.phone) or not self.phone.startswith('+'):
             raise ValidationError(
                 _('Phone does not contain spaces and must be starts with a +'))
 
@@ -177,17 +207,20 @@ class UserAuthNotification(models.Model):
 class UserAuthActivity(models.Model):
     user = models.ForeignKey(
         settings.AUTH_USER_MODEL, editable=False, verbose_name=_('User'))
-    ip = models.CharField(max_length=40)
+    ip = models.CharField(max_length=40, db_index=True)
     geo = models.CharField(_('GEO'), max_length=255, null=True, blank=True)
     date = models.DateTimeField(_('Date'), auto_now_add=True)
     agent = models.CharField(
         _('Browser'), max_length=255, null=True, blank=True)
+    confirm_method = models.CharField(
+        _('Confirm method'), max_length=10, choices=AUTH_TYPES,
+        null=True, blank=True)
 
     class Meta:
         ordering = ('-id',)
 
     @classmethod
-    def log_auth(cls, request):
+    def log_auth(cls, request, confirm_method=''):
         ip = get_ip(request)
         user_agent = request.META.get('HTTP_USER_AGENT')
         if user_agent is not None:
@@ -198,7 +231,7 @@ class UserAuthActivity(models.Model):
                 browser.get('name', ""), browser.get('version', ""))
         cls.objects.create(
             user=request.user, ip=get_ip(request), geo=get_geo(ip),
-            agent=user_agent
+            agent=user_agent, confirm_method=confirm_method
         )
 
     @classmethod
@@ -210,3 +243,47 @@ class UserAuthActivity(models.Model):
                 obj[0].geo, obj[0].ip, obj[0].agent, unicode(
                     _('If it was not you, then change authorization data.')))
             messages.warning(request, show_msg)
+
+
+class UserAuthAttempt(models.Model):
+    ip = models.BigIntegerField(db_index=True)
+    attempt = models.IntegerField(default=0)
+    created = models.DateTimeField(auto_now=True, auto_now_add=True)
+
+    @classmethod
+    def get_obj(cls, request):
+        return cls.objects.get_or_create(ip=inet_aton(get_ip(request)))[0]
+
+    @classmethod
+    def store(cls, request):
+        obj = cls.get_obj(request)
+        obj.attempt = models.F('attempt') + 1
+        obj.save(update_fields=['attempt'])
+
+    @classmethod
+    def remove(cls, request):
+        if CHECK_ATTEMPT is True:
+            obj = cls.get_obj(request)
+            obj.delete()
+
+    @classmethod
+    def is_banned(cls, request):
+        obj = cls.get_obj(request)
+        if obj.attempt > LOGIN_ATTEMPT:
+            if (now() - obj.created).seconds < BAN_TIME:
+                return True
+
+    @classmethod
+    def clean(cls):
+        old = now() - datetime.timedelta(seconds=BAN_TIME)
+        cls.objects.filter(created__lt=old).delete()
+
+
+class UserAuthLogging(models.Model):
+    user = models.OneToOneField(
+        settings.AUTH_USER_MODEL, verbose_name=_('User'))
+    enabled = models.BooleanField(_('Enabled'), default=False)
+
+    @classmethod
+    def is_enabled(cls, request):
+        return cls.objects.filter(user=request.user, enabled=1).exists()
