@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-from django.conf.urls import patterns, url
+from django.conf.urls import patterns, url, include
 
 urlpatterns = patterns(
     'secureauth.views',
@@ -18,4 +18,9 @@ urlpatterns = patterns(
     url('^question_settings/$', 'question_settings', name='question_settings'),
     url('^auth_activity/$', 'auth_activity', name='auth_activity'),
     url('^notify_settings/$', 'notify_settings', name='notify_settings'),
+    url('^logging_settings/$', 'logging_settings', name='logging_settings'),
+    url('^send_codes/$', 'send_codes', name='send_codes'),
+    url('^disable_methods/(\d+)/$', 'disable_methods', name='disable_methods'),
+    url('^unban_ip/$', 'unban_ip', name='unban_ip'),
+    url(r'^captcha/', include('captcha.urls')),
 )
