@@ -1,4 +1,8 @@
-from django.http import HttpResponseRedirect, Http404, HttpResponse
+# -*- coding: utf-8 -*-
+
+from django.contrib.admin.views.decorators import staff_member_required
+from django.http import (
+    HttpResponseRedirect, Http404, HttpResponse, HttpResponseBadRequest)
 from django.utils.translation import ugettext as _
 from django.template.response import TemplateResponse
 from django.utils.http import is_safe_url
@@ -11,7 +15,6 @@ from django.conf import settings
 from django.forms.models import model_to_dict
 from django.contrib.auth import REDIRECT_FIELD_NAME, login as auth_login
 from django.contrib.sites.models import get_current_site
-from django.contrib.auth.forms import AuthenticationForm
 from django.shortcuts import get_object_or_404, redirect
 from django.contrib.auth.decorators import login_required
 from django.core.urlresolvers import reverse
@@ -21,16 +24,18 @@ from django_tables2 import RequestConfig
 
 from secureauth.auth_forms import get_available_auth_methods
 from secureauth.auth_forms import ConfirmAuthenticationForm
+from secureauth.auth_forms import BaseAuthForm
 from secureauth.utils.sign import Sign
 from secureauth.filters import UserAuthActivityFilter
 from secureauth.tables import UserAuthActivityTable
 from secureauth.utils.decorators import ajax_required
 from secureauth.forms import (
-    BasicForm, CodeForm, PhoneBasicForm, QuestionForm, NotificationForm)
+    BasicForm, CodeForm, PhoneBasicForm, QuestionForm, IpBanForm,
+    NotificationForm, LoggingForm, DisableMethodForm)
 from secureauth.models import (
     UserAuthPhone, UserAuthCode, UserAuthQuestion, UserAuthToken,
-    UserAuthActivity, UserAuthNotification)
-from secureauth.defaults import SMS_AGE, SMS_FORCE
+    UserAuthActivity, UserAuthNotification, UserAuthAttempt, UserAuthLogging)
+from secureauth.defaults import SMS_AGE, SMS_FORCE, CHECK_ATTEMPT
 
 
 def _get_data(request):
@@ -48,9 +53,12 @@ def _get_data(request):
 @never_cache
 def login(request, template_name='secureauth/login.html',
           redirect_field_name=REDIRECT_FIELD_NAME,
-          authentication_form=AuthenticationForm,
-          current_app=None, extra_context=None):
-    redirect_to = request.REQUEST.get(redirect_field_name, '')
+          authentication_form=BaseAuthForm,
+          current_app=None, extra_context=None, redirect_to=''):
+    redirect_to = request.REQUEST.get(redirect_field_name, redirect_to)
+
+    if CHECK_ATTEMPT and UserAuthAttempt.is_banned(request):
+        return HttpResponseBadRequest()
 
     if request.method == "POST":
         form = authentication_form(request, data=request.POST)
@@ -69,13 +77,21 @@ def login(request, template_name='secureauth/login.html',
                     'user_pk': user.pk,
                     'credentials': form.cleaned_data,
                     'ip': request.META.get('REMOTE_ADDR'),
+                    'extra_context': extra_context,
                 }
                 data = Sign().sign(data)
                 return HttpResponseRedirect(
                     '%s?data=%s' % (reverse('auth_confirmation'), data))
             else:
                 auth_login(request, user)
+                if UserAuthLogging.is_enabled(request):
+                    UserAuthActivity.check_location(request)
+                    UserAuthActivity.log_auth(request)
+                UserAuthAttempt.remove(request)
                 return HttpResponseRedirect(redirect_to)
+        elif CHECK_ATTEMPT is True:
+            UserAuthAttempt.clean()
+            UserAuthAttempt.store(request)
     else:
         form = authentication_form(request)
 
@@ -101,7 +117,14 @@ def login(request, template_name='secureauth/login.html',
 def login_confirmation(request, template_name='secureauth/confirmation.html',
                        authentication_form=ConfirmAuthenticationForm,
                        extra_context=None, current_app=None):
+
+    if CHECK_ATTEMPT and UserAuthAttempt.is_banned(request):
+        return HttpResponseBadRequest()
+
     data = _get_data(request)
+    if extra_context is None and data.get('extra_context'):
+        extra_context = data.get('extra_context')
+
     if request.method == "POST":
         form = authentication_form(data, request.POST)
         if form.is_valid():
@@ -112,13 +135,20 @@ def login_confirmation(request, template_name='secureauth/confirmation.html',
                 if request.session.test_cookie_worked():
                     request.session.delete_test_cookie()
 
-                UserAuthActivity.check_location(request)
-                UserAuthActivity.log_auth(request)
+                if UserAuthLogging.is_enabled(request):
+                    UserAuthActivity.check_location(request)
+                    UserAuthActivity.log_auth(
+                        request, form.cleaned_data.get('auth_type'))
+
                 UserAuthNotification.notify(request)
+                UserAuthAttempt.remove(request)
 
                 return HttpResponseRedirect(data.get('redirect_to'))
             else:
-                raise Http404('ERROR')
+                return HttpResponseBadRequest()
+        elif CHECK_ATTEMPT is True:
+            UserAuthAttempt.clean()
+            UserAuthAttempt.store(request)
     else:
         form = authentication_form(data)
 
@@ -167,7 +197,8 @@ def get_question(request):
 @login_required
 @never_cache
 def auth_settings(request):
-    UserAuthActivity.check_location(request)
+    if UserAuthLogging.is_enabled(request):
+        UserAuthActivity.check_location(request)
     return render(request, 'secureauth/settings.html')
 
 
@@ -199,6 +230,8 @@ class BasicView(object):
 
     def __do_step(self):
         self.__set_template()
+        if self.step > 1:
+            self._check_step(self.step)
         return getattr(self, self.method_map.get(self.step))()
 
     def _url(self, step):
@@ -210,12 +243,19 @@ class BasicView(object):
         return render(self.request, self.template, self.context)
 
     def _redirect(self, step):
+        self._set_next_step(step)
         return redirect(self._url(step))
 
-    def _get_data(self, model_data=True):
-        if model_data and self.obj.exists() and self.request.method == 'GET':
+    def _get_initial(self):
+        if self.obj.exists() and self.request.method == 'GET':
             return model_to_dict(self.obj[0])
-        return self.request.method == 'POST' and self.request.POST or None
+
+    def _set_next_step(self, step):
+        self.request.session['step'] = step
+
+    def _check_step(self, step):
+        if self.request.session.get('step') != step:
+            raise Http404
 
     def settings_remove(self):
         step = 4 if self.obj.exists() else 1
@@ -239,7 +279,8 @@ class BasicView(object):
 
     def settings(self):
         self.form = self.basic_form(
-            self.request.user, self.model, self._get_data())
+            self.request, self.model, self.request.POST or None,
+            initial=self._get_initial())
         if self.request.method == 'POST':
             if self.form.is_valid():
                 form_enabled = self.form.cleaned_data.get('enabled')
@@ -256,7 +297,7 @@ class BasicView(object):
         # todo: check ip there & referer with key.
         # Do not open this page with GET url
         self.form = self.code_form(
-            self.request.user, self.model, self._get_data(False))
+            self.request.user, self.model, self.request.POST or None)
         if self.request.method == 'POST':
             if self.form.is_valid():
                 self.form.save()
@@ -299,6 +340,19 @@ def codes_settings(request):
     return view.get()
 
 
+@login_required
+@never_cache
+def send_codes(request):
+    if request.session.get('step') != 3:
+        raise Http404
+    elif UserAuthCode.send_codes(request):
+        messages.info(request, _('Codes were sent to the email'))
+        UserAuthNotification.notify(request, _('Codes were sent to the email'))
+        if request.session.get('step'):
+            del request.session['step']
+    return redirect('codes_settings')
+
+
 @login_required
 @never_cache
 def question_settings(request):
@@ -311,11 +365,9 @@ def question_settings(request):
 @login_required
 @never_cache
 def auth_activity(request):
-    queryset = UserAuthActivity.objects.select_related().filter(
-        user=request.user)
+    queryset = UserAuthActivity.objects.filter(user=request.user)
     queryset = UserAuthActivityFilter(request.GET, queryset=queryset)
     table = UserAuthActivityTable(queryset)
-    # table.paginate(page=request.GET.get('page', 1))
     RequestConfig(request).configure(table)
     return render(request, 'secureauth/auth_activity.html', {
         'table': table,
@@ -323,19 +375,62 @@ def auth_activity(request):
     })
 
 
-@login_required
-@never_cache
-def notify_settings(request):
-    instance = UserAuthNotification.objects.get_or_create(user=request.user)[0]
-    data = request.method == 'POST' and request.POST or model_to_dict(instance)
-    form = NotificationForm(data, instance=instance)
-    if request.method == 'POST':
-        if form.is_valid():
+def _settings_view(request, model_class, form_class, template):
+    instance = model_class.objects.get_or_create(user=request.user)[0]
+    data = request.POST or None
+    form = form_class(request, data, instance=instance)
+    if request.method == 'POST' and form.is_valid():
             form.save(commit=False)
             form.user = request.user
             form.save()
             messages.info(request, _('Successfully saved'))
             if not form.cleaned_data.get('enabled'):
                 UserAuthNotification.notify(
-                    request, _('Notification was disabled'), force=True)
-    return render(request, 'secureauth/notify_settings.html', {'form': form})
+                    request, _('Your settings has changed'), force=True)
+    return render(request, template, {'form': form})
+
+
+@login_required
+@never_cache
+def notify_settings(request):
+    return _settings_view(
+        request, UserAuthNotification, NotificationForm,
+        'secureauth/notify_settings.html'
+    )
+
+
+@login_required
+@never_cache
+def logging_settings(request):
+    return _settings_view(
+        request, UserAuthLogging, LoggingForm,
+        'secureauth/logging_settings.html'
+    )
+
+
+@staff_member_required
+@login_required
+@never_cache
+def disable_methods(request, pk):
+    form = DisableMethodForm(request, pk, request.POST or None)
+    if request.method == 'POST' and form.is_valid():
+        messages.info(request, _('Successfully saved'))
+        form.save()
+        return redirect('disable_methods', pk)
+    return render(request, 'secureauth/admin_disable_methods.html', {
+        'form': form, 'pk': pk
+    })
+
+
+@staff_member_required
+@login_required
+@never_cache
+def unban_ip(request):
+    form = IpBanForm(request.POST or None)
+    if request.method == 'POST' and form.is_valid():
+        messages.info(request, _('Successfully saved'))
+        form.save()
+        return redirect('unban_ip')
+    return render(request, 'secureauth/admin_ban_ip.html', {
+        'form': form
+    })
